unit uGate10I4O;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses uGate, uBTree, Graphics;

const
  Ins = 10;
  Outs = 4;

type
  // TGate10I4O AKA The Copy&Paste Job
  TGate10I4O = class(TGate)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure setIA(n : Integer; i : TPowerLine); override;
      procedure setOA(n : Integer; o : TTree); override;
    protected
      IA : array [1..Ins] of TPowerLine;
      VA : array [1..Ins] of Boolean;
      OA : array [1..Outs] of TTree;
      procedure setVA(n : Integer; v : Boolean); override;
      function getIA(n : Integer) : TPowerLine; override;
      function getOA(n : Integer) : TTree; override;
  end;
  T4bRAM = class(TGate10I4O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      constructor loadFromString(s : String; c : TCanvas); override;
      procedure recalculateValues; override;
      function getDataString : String; override;
    protected
      Data : array [0..15, 0..3] of Boolean;
  end;

implementation

// T4bRAM
//----------------------------------------------------------------------
constructor T4bRAM.Init(x, y : Integer; c : TCanvas; i : Integer);
var
  JackHeight, j, k : Integer;
begin
  inherited Init(x, y, c, i);
  for j := 0 to 15 do
    for k := 0 to 3 do
      Data[j, k] := false;
  JackHeight := (H div self.I) + 1;
  with ownPicture.Canvas do
    begin
      Pen.Color := clLtGray;
      MoveTo(1, H - 3);
      LineTo(10, H - 3);
      Pen.Color := clBlack;
      TextOut(13, 3, 'RAM');
      // Input descriptions.
      Font.Name := 'Tahoma';
      Font.Height := -7;
      TextOut(4, 1 + JackHeight * 0, 'F');
      TextOut(3, JackHeight * 1, 'W');
      TextOut(4, JackHeight * 2 - 1, '0');
      TextOut(4, JackHeight * 3 - 1, '|');
      TextOut(4, JackHeight * 4 - 1, '2');
      TextOut(4, JackHeight * 5 - 2, '3');
      TextOut(4, JackHeight * 6 - 3, 'A');
      TextOut(4, JackHeight * 7 - 4, 'B');
      TextOut(4, JackHeight * 8 - 5, 'C');
      TextOut(4, JackHeight * 9 - 6, 'D');
      JackHeight := (H div O) + 1;
      TextOut(W - 7, 10 + JackHeight * 0, '0');
      TextOut(W - 7, 6 + JackHeight * 1, '|');
      TextOut(W - 7, 4 + JackHeight * 2, '2');
      TextOut(W - 7, 2 + JackHeight * 3, '3');
    end;
end;

constructor T4bRAM.loadFromString(s : String; c : TCanvas);
var
  j, k : Integer;
begin
  inherited loadFromString(s, c);
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  for j := 0 to 15 do
    for k := 0 to 3 do
      begin
        if Copy(s, 1, 1) = '1' then
          Data[j, k] := true
        else
          Data[j, k] := false;
        Delete(s, 1, 2);
      end;
end;

procedure T4bRAM.recalculateValues;
var
  Address : Integer;
begin
  if VA[1] then exit;
  Address := 0;
  if VA[3] then
    Address := 1;
  if VA[4] then
    Address := Address + 2;
  if VA[5] then
    Address := Address + 4;
  if VA[6] then
    Address := Address + 8;
  if VA[2] then
    begin
      Data[Address, 0] := VA[7];
      Data[Address, 1] := VA[8];
      Data[Address, 2] := VA[9];
      Data[Address, 3] := VA[10];
      exit;
    end;
  notifyOutputJack(1, Data[Address, 0]);
  notifyOutputJack(2, Data[Address, 1]);
  notifyOutputJack(3, Data[Address, 2]);
  notifyOutputJack(4, Data[Address, 3]);
end;

function T4bRAM.getDataString : String;
var
  FieldString, DataString : String;
  j, k, MobEndPosition : Integer;
begin
  FieldString := '';
  for j := 0 to 15 do
    for k := 0 to 3 do
      if Data[j, k] then
        FieldString := FieldString + ':1'
      else
        FieldString := FieldString + ':0';
  DataString := inherited getDataString;
  if Pos('|', DataString) > 0 then
    MobEndPosition := Pos('|', DataString)
  else
    MobEndPosition := Length(DataString) + 1;
  Insert(FieldString, DataString, MobEndPosition);
  result := DataString;
end;

// TGate10I4O
//----------------------------------------------------------------------
constructor TGate10I4O.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Create(x, y, Ins, Outs, c);
  Id := i;
end;

procedure TGate10I4O.setIA(n : Integer; i : TPowerLine);
begin
  IA[n] := i;
end;

procedure TGate10I4O.setOA(n : Integer; o : TTree);
begin
  OA[n] := o;
end;

procedure TGate10I4O.setVA(n : Integer; v : Boolean);
begin
  VA[n] := v;
end;

function TGate10I4O.getIA(n : Integer) : TPowerLine;
begin
  result := IA[n];
end;

function TGate10I4O.getOA(n : Integer) : TTree;
begin
  result := OA[n];
end;

end.
