unit uGate1I;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses uGate, uBTree, Graphics;

const
  Ins = 1;
  Outs = 0;

type
  // TGate1I AKA The Copy&Paste Job
  TGate1I = class(TGate)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure setIA(n : Integer; i : TPowerLine); override;
      procedure setOA(n : Integer; o : TTree); override;
    protected
      IA : array [1..Ins] of TPowerLine;
      VA : array [1..Ins] of Boolean;
      procedure setVA(n : Integer; v : Boolean); override;
      function getIA(n : Integer) : TPowerLine; override;
      function getOA(n : Integer) : TTree; override;
  end;
  TLED = class(TGate1I)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      constructor loadFromString(s : String; c : TCanvas); override;
      procedure recalculateValues; override;
      function getDataString : String; override;
    private
      State : Boolean;
  end;


implementation

// TLED
//----------------------------------------------------------------------
constructor TLED.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Init(x, y, c, i);
  State := false;
  recalculateValues;
end;

constructor TLED.loadFromString(s : String; c : TCanvas);
begin
  inherited loadFromString(s, c);
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  if Copy(s, 1, 1) = '1' then
    State := true
  else
    State := false;
  with ownPicture.Canvas do
    begin
      if State then
        Brush.Color := clRed
      else
        Brush.Color := clMaroon;
      Pen.Color := clBlack;
      Ellipse(15, (H div 2) - 15, W - 5, (H div 2) + 15);
    end;
  blitToCanvas;
end;

procedure TLED.recalculateValues;
begin
  State := VA[1];
  with ownPicture.Canvas do
    begin
      if State then
        Brush.Color := clRed
      else
        Brush.Color := clMaroon;
      Pen.Color := clBlack;
      Ellipse(15, (H div 2) - 15, W - 5, (H div 2) + 15);
    end;
end;

function TLED.getDataString : String;
var
  DataString, FieldString : String;
  MobEndPosition : Integer;
begin
  if State then
    FieldString := ':1'
  else
    FieldString := ':0';
  DataString := inherited getDataString;
  if Pos('|', DataString) > 0 then
    MobEndPosition := Pos('|', DataString)
  else
    MobEndPosition := Length(DataString) + 1;
  Insert(FieldString, DataString, MobEndPosition);
  result := DataString;
end;

// TGate1I
//----------------------------------------------------------------------
constructor TGate1I.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Create(x, y, Ins, Outs, c);
  Id := i;
end;

procedure TGate1I.setIA(n : Integer; i : TPowerLine);
begin
  IA[n] := i;
end;

procedure TGate1I.setOA(n : Integer; o : TTree);
begin
end;

procedure TGate1I.setVA(n : Integer; v : Boolean);
begin
  VA[n] := v;
end;

function TGate1I.getIA(n : Integer) : TPowerLine;
begin
  result := IA[n];
end;

function TGate1I.getOA(n : Integer) : TTree;
begin
  result := nil;
end;

end.
