unit uGate1O;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses uGate, uBTree, Graphics, Classes, Controls, ExtCtrls;

const
  Ins = 0;
  Outs = 1;

type
  // TGate1O AKA The Copy&Paste Job
  TGate1O = class(TGate)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure setIA(n : Integer; i : TPowerLine); override;
      procedure setOA(n : Integer; o : TTree); override;
    protected
      OA : array [1..Outs] of TTree;
      procedure setVA(n : Integer; v : Boolean); override;
      function getIA(n : Integer) : TPowerLine; override;
      function getOA(n : Integer) : TTree; override;
  end;
  TSwitch = class(TGate1O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      constructor loadFromString(s : String; c : TCanvas); override;
      procedure MouseUp(x, y : Integer; m : TMouseButton); override;
      procedure recalculateValues; override;
      procedure drawState;
      function getDataString : String; override;
    private
      State : Boolean;
  end;
  TSignalPulse = class(TGate1O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      constructor loadFromString(s : String; c : TCanvas); override;
      destructor Destroy; override;
      procedure MouseUp(x, y : Integer; m : TMouseButton); override;
      procedure recalculateValues; override;
      procedure changeState(Sender: TObject);
      function getDataString : String; override;
    private
      State : Boolean;
      PulseTimer : TTimer;
  end;

implementation

// TSignalPulse
//----------------------------------------------------------------------
constructor TSignalPulse.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Init(x, y, c, i);
  State := true;
  PulseTimer := TTimer.Create(self);
  PulseTimer.OnTimer := self.changeState;
  PulseTimer.Enabled := false;
  PulseTimer.Interval := 500;
  with ownPicture.Canvas do
    begin
      Brush.Color := clGreen;
      Pen.Color := clBlack;
      Rectangle(6, 5, W - 14, 15);
    end;
  changeState(self);
end;

constructor TSignalPulse.loadFromString(s : String; c : TCanvas);
begin
  inherited loadFromString(s, c);
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  if Copy(s, 1, 1) = '1' then
    State := true
  else
    State := false;
  with ownPicture.Canvas do
    begin
      if State then
        Brush.Color := clRed
      else
        Brush.Color := clMaroon;
      Pen.Color := clBlack;
      Ellipse(6, (H div 2) - 15, W - 14, (H div 2) + 15);
    end;
  blitToCanvas;
end;

destructor TSignalPulse.Destroy;
begin
  PulseTimer.Free;
  inherited Destroy;
end;

procedure TSignalPulse.MouseUp(x, y : Integer; m : TMouseButton);
begin
  inherited MouseUp(x, y, m);
  if (getClickedSection(x, y).Output = 0) and (m = mbLeft) then PulseTimer.Enabled := Not(PulseTimer.Enabled);
  with ownPicture.Canvas do
    begin
      if PulseTimer.Enabled then
        Brush.Color := clLime
      else
        Brush.Color := clGreen;
      Pen.Color := clBlack;
      Rectangle(6, 5, W - 14, 15);
    end;
end;

procedure TSignalPulse.recalculateValues;
begin
  notifyOutputJack(1, State);
end;

procedure TSignalPulse.changeState(Sender: TObject);
begin
  State := Not(State);
  with ownPicture.Canvas do
    begin
      if State then
        Brush.Color := clRed
      else
        Brush.Color := clMaroon;
      Pen.Color := clBlack;
      Ellipse(6, (H div 2) - 15, W - 14, (H div 2) + 15);
    end;
  blitToCanvas;
  notifyOutputJack(1, State);
end;

function TSignalPulse.getDataString : String;
var
  DataString, FieldString : String;
  MobEndPosition : Integer;
begin
  if State then
    FieldString := ':1'
  else
    FieldString := ':0';
  DataString := inherited getDataString;
  if Pos('|', DataString) > 0 then
    MobEndPosition := Pos('|', DataString)
  else
    MobEndPosition := Length(DataString) + 1;
  Insert(FieldString, DataString, MobEndPosition);
  result := DataString;
end;


// TSwitch
//----------------------------------------------------------------------
constructor TSwitch.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Init(x, y, c, i);
  State := false;
  drawState;
end;

constructor TSwitch.loadFromString(s : String; c : TCanvas);
begin
  inherited loadFromString(s, c);
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  if Copy(s, 1, 1) = '1' then
    State := true
  else
    State := false;
  drawState;
  blitToCanvas;
end;

procedure TSwitch.recalculateValues;
begin
  notifyOutputJack(1, State);
end;

procedure TSwitch.drawState;
begin
  with ownPicture.Canvas do
    begin
      Brush.Color := clLtGray;
      Pen.Color := clBlack;
      FillRect(Rect(15, 1, W - 10, H - 2));
      TextOut(15, H div 4, '1');
      if State then
        LineTo(W - 10, H div 2)
      else
        LineTo(16, H div 4);
      TextOut(15, 3 * (H div 4), '0');
      if Not(State) then
        LineTo(W - 10, H div 2)
      else
        LineTo(16, 3 * (H div 4));
    end;
end;

procedure TSwitch.MouseUp(x, y: Integer; m : TMouseButton);
begin
  inherited MouseUp(x, y, m);
  if (getClickedSection(x, y).Output = 0) and (m = mbLeft) then
    begin
      State := Not(State);
      drawState;
      notifyOutputJack(1, State);
      blitToCanvas;
    end;
end;

function TSwitch.getDataString : String;
var
  DataString, FieldString : String;
  MobEndPosition : Integer;
begin
  if State then
    FieldString := ':1'
  else
    FieldString := ':0';
  DataString := inherited getDataString;
  if Pos('|', DataString) > 0 then
    MobEndPosition := Pos('|', DataString)
  else
    MobEndPosition := Length(DataString) + 1;
  Insert(FieldString, DataString, MobEndPosition);
  result := DataString;
end;

// TGate1O
//----------------------------------------------------------------------
constructor TGate1O.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Create(x, y, Ins, Outs, c);
  Id := i;
end;

procedure TGate1O.setIA(n : Integer; i : TPowerLine);
begin
end;

procedure TGate1O.setOA(n : Integer; o : TTree);
begin
  OA[n] := o;
end;

procedure TGate1O.setVA(n : Integer; v : Boolean);
begin
end;

function TGate1O.getIA(n : Integer) : TPowerLine;
begin
  result := nil;
end;

function TGate1O.getOA(n : Integer) : TTree;
begin
  result := OA[n];
end;

end.
