unit uGate2I1O;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses uGate, uBTree, Graphics;

const
  Ins = 2;
  Outs = 1;

type
  // TGate2I1O AKA The Copy&Paste Job
  TGate2I1O = class(TGate)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure setIA(n : Integer; i : TPowerLine); override;
      procedure setOA(n : Integer; o : TTree); override;
    protected
      IA : array [1..Ins] of TPowerLine;
      VA : array [1..Ins] of Boolean;
      OA : array [1..Outs] of TTree;
      procedure setVA(n : Integer; v : Boolean); override;
      function getIA(n : Integer) : TPowerLine; override;
      function getOA(n : Integer) : TTree; override;
  end;
  TNand = class(TGate2I1O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure recalculateValues; override;
  end;
  TNor = class(TGate2I1O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure recalculateValues; override;
  end;
  TAnd = class(TGate2I1O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure recalculateValues; override;
  end;
  TOr = class(TGate2I1O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure recalculateValues; override;
  end;
  TXor = class(TGate2I1O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure recalculateValues; override;
  end;

implementation

// TNor
//----------------------------------------------------------------------
constructor TNor.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Init(x, y, c, i);
  ownPicture.Canvas.TextOut(15, 5, 'Nor');
end;

procedure TNor.recalculateValues;
begin
  notifyOutputJack(1, Not(VA[1] or VA[2]));
end;

// TNand
//----------------------------------------------------------------------
constructor TNand.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Init(x, y, c, i);
  ownPicture.Canvas.TextOut(12, 5, 'Nand');
end;

procedure TNand.recalculateValues;
begin
  notifyOutputJack(1, Not(VA[1] and VA[2]));
end;

// TOr
//----------------------------------------------------------------------
constructor TOr.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Init(x, y, c, i);
  ownPicture.Canvas.TextOut(15, 5, 'Or');
end;

procedure TOr.recalculateValues;
begin
  notifyOutputJack(1, VA[1] or VA[2]);
end;

// TXor
//----------------------------------------------------------------------
constructor TXor.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Init(x, y, c, i);
  ownPicture.Canvas.TextOut(15, 5, 'Xor');
end;

procedure TXor.recalculateValues;
begin
  notifyOutputJack(1, VA[1] xor VA[2]);
end;

// TAnd
//----------------------------------------------------------------------
constructor TAnd.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Init(x, y, c, i);
  ownPicture.Canvas.TextOut(15, 5, 'And');
end;

procedure TAnd.recalculateValues;
begin
  notifyOutputJack(1, VA[1] and VA[2]);
end;

// TGate1I1O
//----------------------------------------------------------------------
constructor TGate2I1O.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Create(x, y, Ins, Outs, c);
  Id := i;
end;

procedure TGate2I1O.setIA(n : Integer; i : TPowerLine);
begin
  IA[n] := i;
end;

procedure TGate2I1O.setOA(n : Integer; o : TTree);
begin
  OA[n] := o;
end;

procedure TGate2I1O.setVA(n : Integer; v : Boolean);
begin
  VA[n] := v;
end;

function TGate2I1O.getIA(n : Integer) : TPowerLine;
begin
  result := IA[n];
end;

function TGate2I1O.getOA(n : Integer) : TTree;
begin
  result := OA[n];
end;

end.
