unit uGate3I3O;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses uGate, uBTree, Graphics;

const
  Ins = 3;
  Outs = 3;

type
  // TGate3I3O AKA The Copy&Paste Job
  TGate3I3O = class(TGate)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure setIA(n : Integer; i : TPowerLine); override;
      procedure setOA(n : Integer; o : TTree); override;
    protected
      IA : array [1..Ins] of TPowerLine;
      VA : array [1..Ins] of Boolean;
      OA : array [1..Outs] of TTree;
      procedure setVA(n : Integer; v : Boolean); override;
      function getIA(n : Integer) : TPowerLine; override;
      function getOA(n : Integer) : TTree; override;
  end;
  TOneBitFullAdder = class(TGate3I3O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure recalculateValues; override;
  end;

implementation

// TOneBitFullAdder
//----------------------------------------------------------------------
constructor TOneBitFullAdder.Init(x, y : Integer; c : TCanvas; i : Integer);
var
  JackHeight : Integer;
begin
  inherited Init(x, y, c, i);
  JackHeight := (H div self.I) + 1;
  with ownPicture.Canvas do
    begin
      TextOut(12, 3, '1BFA');
      TextOut(12, 15 + JackHeight * 0, 'A');
      TextOut(12, 8 + JackHeight * 1, 'B');
      TextOut(12, 6 + JackHeight * 2, 'C');
      TextOut(28, 15 + JackHeight * 0, 'C');
      TextOut(28, 8 + JackHeight * 1, 'S');
      TextOut(28, 6 + JackHeight * 2, '!S');
    end;
end;

procedure TOneBitFullAdder.recalculateValues;
begin
  notifyOutputJack(1, (VA[1] and (VA[2] or VA[3])) or (VA[2] and VA[3]));
  notifyOutputJack(2, ((VA[1] xor VA[2]) xor VA[3]) or (VA[1] and VA[2] and VA[3]));
  notifyOutputJack(3, Not(((VA[1] xor VA[2]) xor VA[3]) or (VA[1] and VA[2] and VA[3])));
end;

// TGate3I3O
//----------------------------------------------------------------------
constructor TGate3I3O.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Create(x, y, Ins, Outs, c);
  Id := i;
end;

procedure TGate3I3O.setIA(n : Integer; i : TPowerLine);
begin
  IA[n] := i;
end;

procedure TGate3I3O.setOA(n : Integer; o : TTree);
begin
  OA[n] := o;
end;

procedure TGate3I3O.setVA(n : Integer; v : Boolean);
begin
  VA[n] := v;
end;

function TGate3I3O.getIA(n : Integer) : TPowerLine;
begin
  result := IA[n];
end;

function TGate3I3O.getOA(n : Integer) : TTree;
begin
  result := OA[n];
end;

end.
