unit uGate5I2O;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses uGate, uBTree, Graphics;

const
  Ins = 5;
  Outs = 2;
                         
type
  // TGate5I2O AKA The Copy&Paste Job
  TGate5I2O = class(TGate)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure setIA(n : Integer; i : TPowerLine); override;
      procedure setOA(n : Integer; o : TTree); override;
    protected
      IA : array [1..Ins] of TPowerLine;
      VA : array [1..Ins] of Boolean;
      OA : array [1..Outs] of TTree;
      procedure setVA(n : Integer; v : Boolean); override;
      function getIA(n : Integer) : TPowerLine; override;
      function getOA(n : Integer) : TTree; override;
  end;
  TFlipflop = class(TGate5I2O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      constructor loadFromString(s : String; c : TCanvas); override;
      procedure recalculateValues; override;
      procedure drawState;
      function getDataString : String; override;
    protected
      OldPulseSignal, PreState, State : Boolean;
  end;

implementation

// TFlipflop
//----------------------------------------------------------------------
constructor TFlipflop.Init(x, y : Integer; c : TCanvas; i : Integer);
var
  JackHeight : Integer;
begin
  inherited Init(x, y, c, i);
  OldPulseSignal := true;
  PreState := true;
  State := false;
  JackHeight := (H div self.I) + 1;
  with ownPicture.Canvas do
    begin
      //TextOut(23, 3, 'FF');
      TextOut(12, 5 + JackHeight * 0, '!S');
      TextOut(12, 3 + JackHeight * 1, '1J');
      TextOut(12, 2 + JackHeight * 2, '>C1');
      TextOut(12, 1 + JackHeight * 3, '1K');
      TextOut(12, JackHeight * 4, '!R');
      TextOut(30, JackHeight div 2 + 5, 'Q');
      TextOut(28, 3 * JackHeight + JackHeight div 2, '!Q');
    end;
  drawState;
  blitToCanvas;
end;

constructor TFlipFlop.loadFromString(s : String; c : TCanvas);
begin
  inherited loadFromString(s, c);
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  Delete(s, 1, Pos(':', s));
  if Copy(s, 1, 1) = '1' then
    OldPulseSignal := true
  else
    OldPulseSignal := false;
  Delete(s, 1, 2);
  if Copy(s, 1, 1) = '1' then
    PreState := true
  else
    PreState := false;
  Delete(s, 1, 2);
  if Copy(s, 1, 1) = '1' then
    State := true
  else
    State := false;
  drawState;
  blitToCanvas;
end;

procedure TFlipflop.recalculateValues;
begin
  if VA[1] and Not(VA[5]) then
    begin
      State := false;
      notifyOutputJack(1, false);
      notifyOutputJack(2, true);
      drawState;
      exit;
    end;
  if Not(VA[1]) and VA[5] then
    begin
      State := true;
      notifyOutputJack(1, true);
      notifyOutputJack(2, false);
      drawState;
      exit;
    end;
  if OldPulseSignal and Not(VA[3]) then
    begin
      if VA[2] and VA[4] then
        begin
          OldPulseSignal := VA[3];
          State := PreState;
          notifyOutputJack(1, PreState);
          PreState := Not(PreState);
          notifyOutputJack(2, PreState);
          drawState;
          exit;
        end;
      State := PreState;
      notifyOutputJack(1, PreState);
      notifyOutputJack(2, Not(PreState));
    end;
  if VA[2] xor VA[4] then PreState := VA[2];
  OldPulseSignal := VA[3];
  drawState;
end;

procedure TFlipflop.drawState;
begin
  with ownPicture.Canvas do
    begin
      if State then
        Brush.Color := clRed
      else
        Brush.Color := clMaroon;
      Pen.Color := clBlack;
      Rectangle(W div 2, 3, W - 12, 7);
    end;
end;

function TFlipflop.getDataString : String;
var
  DataString, SOldPulseSignal, SPreState, SState : String;
  MobEndPosition : Integer;
begin
  SOldPulseSignal := '0';
  SPreState := '0';
  SState := '0';
  if OldPulseSignal then
    SOldPulseSignal := '1';
  if PreState then
    SPreState := '1';
  if State then
    SState := '1';
  DataString := inherited getDataString;
  if Pos('|', DataString) > 0 then
    MobEndPosition := Pos('|', DataString)
  else
    MobEndPosition := Length(DataString) + 1;
  Insert(':' + SOldPulseSignal + ':' + SPreState + ':' + SState, DataString, MobEndPosition);
  result := DataString;
end;

// TGate5I2O
//----------------------------------------------------------------------
constructor TGate5I2O.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Create(x, y, Ins, Outs, c);   
  Id := i;
end;

procedure TGate5I2O.setIA(n : Integer; i : TPowerLine);
begin
  IA[n] := i;
end;

procedure TGate5I2O.setOA(n : Integer; o : TTree);
begin
  OA[n] := o;
end;

procedure TGate5I2O.setVA(n : Integer; v : Boolean);
begin
  VA[n] := v;
end;

function TGate5I2O.getIA(n : Integer) : TPowerLine;
begin
  result := IA[n];
end;

function TGate5I2O.getOA(n : Integer) : TTree;
begin
  result := OA[n];
end;

end.
