unit uGate6I1O;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses uGate, uBTree, Graphics;

const
  Ins = 6;
  Outs = 1;

type
  // TGate6I1O AKA The Copy&Paste Job
  TGate6I1O = class(TGate)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure setIA(n : Integer; i : TPowerLine); override;
      procedure setOA(n : Integer; o : TTree); override;
    protected
      IA : array [1..Ins] of TPowerLine;
      VA : array [1..Ins] of Boolean;
      OA : array [1..Outs] of TTree;
      procedure setVA(n : Integer; v : Boolean); override;
      function getIA(n : Integer) : TPowerLine; override;
      function getOA(n : Integer) : TTree; override;
  end;
  T41Multiplexer = class(TGate6I1O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure recalculateValues; override;
  end;

implementation

// T41Multiplexer
//----------------------------------------------------------------------
constructor T41Multiplexer.Init(x, y : Integer; c : TCanvas; i : Integer);
var
  JackHeight : Integer;
begin
  inherited Init(x, y, c, i);
  JackHeight := (H div self.I) + 1;
  with ownPicture.Canvas do
    begin
      Pen.Color := clLtGray;
      MoveTo(1, H - 4);
      LineTo(10, H - 4);
      Pen.Color := clBlack;
      TextOut(13, 5, 'MUX');
      TextOut(2, JackHeight * 0 + 2, '0');
      TextOut(2, JackHeight * 1 + 1, '1');
      TextOut(2, JackHeight * 2 + 1, 'A');
      TextOut(2, JackHeight * 3 - 1, 'B');
      TextOut(2, JackHeight * 4 - 2, 'C');
      TextOut(2, JackHeight * 5 - 3, 'D');
    end;
end;

procedure T41Multiplexer.recalculateValues;
var
  Base10 : Integer;
begin
  Base10 := 0;
  if VA[1] then
    Base10 := 1;
  if VA[2] then
    Base10 := Base10 + 2;
  notifyOutputJack(1, VA[3+Base10]);
end;

// TGate6I1O
//----------------------------------------------------------------------
constructor TGate6I1O.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Create(x, y, Ins, Outs, c);
  Id := i;
end;

procedure TGate6I1O.setIA(n : Integer; i : TPowerLine);
begin
  IA[n] := i;
end;

procedure TGate6I1O.setOA(n : Integer; o : TTree);
begin
  OA[n] := o;
end;

procedure TGate6I1O.setVA(n : Integer; v : Boolean);
begin
  VA[n] := v;
end;

function TGate6I1O.getIA(n : Integer) : TPowerLine;
begin
  result := IA[n];
end;

function TGate6I1O.getOA(n : Integer) : TTree;
begin
  result := OA[n];
end;

end.
