unit uGate9I5O;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses uGate, uBTree, Graphics;

const
  Ins = 9;
  Outs = 5;

type
  // TGate9I5O AKA The Copy&Paste Job
  TGate9I5O = class(TGate)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure setIA(n : Integer; i : TPowerLine); override;
      procedure setOA(n : Integer; o : TTree); override;
    protected
      IA : array [1..Ins] of TPowerLine;
      VA : array [1..Ins] of Boolean;
      OA : array [1..Outs] of TTree;
      procedure setVA(n : Integer; v : Boolean); override;
      function getIA(n : Integer) : TPowerLine; override;
      function getOA(n : Integer) : TTree; override;
  end;
  TFourBitFullAdder = class(TGate9I5O)
    public
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); override;
      procedure recalculateValues; override;
  end;

implementation

// TFourBitFullAdder
//----------------------------------------------------------------------
constructor TFourBitFullAdder.Init(x, y : Integer; c : TCanvas; i : Integer);
var
  JackHeight : Integer;
begin
  inherited Init(x, y, c, i);
  JackHeight := (H div self.I) + 1;
  with ownPicture.Canvas do
    begin
      Pen.Color := clLtGray;
      MoveTo(1, H - 3);
      LineTo(10, H - 3);
      Pen.Color := clBlack;
      TextOut(12, 3, '4BFA');
      // Input descriptions.
      Font.Name := 'Tahoma';
      Font.Height := -7;
      TextOut(4, 1 + JackHeight * 0, '|');
      TextOut(4, JackHeight * 1, '2');
      TextOut(4, JackHeight * 2 - 1, '3');
      TextOut(4, JackHeight * 3 - 1, '4');
      TextOut(4, JackHeight * 4 - 1, 'C');
      TextOut(3, JackHeight * 5 - 2, 'W');
      TextOut(4, JackHeight * 6 - 3, 'X');
      TextOut(4, JackHeight * 7 - 4, 'Y');
      TextOut(4, JackHeight * 8 - 5, 'Z');
      JackHeight := (H div O) + 1;
      TextOut(W - 7, 10 + JackHeight * 0, 'C');
      TextOut(W - 7, 6 + JackHeight * 1, '|');
      TextOut(W - 7, 4 + JackHeight * 2, '2');
      TextOut(W - 7, 2 + JackHeight * 3, '3');
      TextOut(W - 7, JackHeight * 4, '4');
    end;
end;

procedure TFourBitFullAdder.recalculateValues;
var
  j, Base10 : Integer;
  IVA : array [1..9] of Integer;
  OVA : array [1..5] of Boolean;
begin
  for j := 1 to 9 do
    if VA[j] then
      IVA[j] := 1
    else
      IVA[j] := 0;
  Base10 := IVA[5] + IVA[1] + IVA[6] + IVA[2]* 2 + IVA[7] * 2 + IVA[3] * 4 + IVA[8] * 4 + IVA[4] * 8 + IVA[9] * 8;
  OVA[1] := Boolean(Base10 div 16);
  Base10 := Base10 mod 16;
  OVA[5] := Boolean(Base10 div 8);
  Base10 := Base10 mod 8;
  OVA[4] := Boolean(Base10 div 4);
  Base10 := Base10 mod 4;
  OVA[3] := Boolean(Base10 div 2);
  Base10 := Base10 mod 2;
  OVA[2] := Boolean(Base10);
  for j := 1 to 5 do
    notifyOutputJack(j, OVA[j]);
end;

// TGate9I5O
//----------------------------------------------------------------------
constructor TGate9I5O.Init(x, y : Integer; c : TCanvas; i : Integer);
begin
  inherited Create(x, y, Ins, Outs, c);
  Id := i;
end;

procedure TGate9I5O.setIA(n : Integer; i : TPowerLine);
begin
  IA[n] := i;
end;

procedure TGate9I5O.setOA(n : Integer; o : TTree);
begin
  OA[n] := o;
end;

procedure TGate9I5O.setVA(n : Integer; v : Boolean);
begin
  VA[n] := v;
end;

function TGate9I5O.getIA(n : Integer) : TPowerLine;
begin
  result := IA[n];
end;

function TGate9I5O.getOA(n : Integer) : TTree;
begin
  result := OA[n];
end;

end.
