unit uMovableObject;
{
    This file is part of Dizzy.

    Dizzy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dizzy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dizzy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
// TMob is a generic movable image with event handlers. It has it's own canvas
// and blits it onto the target canvas.

interface

uses Classes, Controls, Graphics;

type
  TMob = class (TComponent)
    public
      constructor Create(x, y, w, h : Integer; c : TCanvas); virtual;
      constructor Init(x, y : Integer; c : TCanvas; i : Integer); virtual; abstract;
      constructor loadFromString(s : String; c : TCanvas); virtual; abstract;
      // Drawing procedure.
      procedure blitToCanvas; virtual;
      // This is the event-handling interface.
      function targetMouseEvent(x, y : Integer; m : TMouseButton; eventType : shortint) : boolean;
    protected
      X, Y, W, H : Integer;
      targetCanvas : TCanvas;
      ownPicture : TBitmap;
      isClicked : boolean;
      // Event handlers.
      procedure MouseDown(x, y : Integer; m : TMouseButton); virtual;
      procedure MouseMove(x, y : Integer; m : TMouseButton); virtual;
      procedure MouseUp(x, y : Integer; m : TMouseButton); virtual;
  end;
  TMobClass = class of TMob;

implementation

constructor TMob.Create(x, y, w, h : Integer; c : TCanvas);
begin
  self.X := x;
  self.Y := y;
  self.W := w;
  self.H := h;
  targetCanvas := c;
  isClicked := false;
  ownPicture := TBitmap.Create;
  ownPicture.Width := self.W;
  ownPicture.Height := self.H;
end;

procedure TMob.blitToCanvas;
begin
  targetCanvas.CopyRect(Rect(X, Y, X + W, Y + H), ownPicture.Canvas, Rect(0, 0, W, H));
end;

function TMob.targetMouseEvent(x, y : Integer; m : TMouseButton; eventType : shortint) : boolean;
begin
  result := false;
  if (x >= self.X) and (y >= self.Y) and (x <= self.X + W) and (y <= self.Y + H) then
    begin
      result := true;
      case eventType of
        0: MouseDown(x, y, m);
        1: MouseMove(x, y, m);
        2: MouseUp(x, y, m);
      end;
    end;
end;

procedure TMob.MouseDown(x, y : Integer; m : TMouseButton);
begin
  if m = mbRight then isClicked := Not(isClicked);
  if Not(isClicked) then exit;
  self.X := x - (W div 2);
  self.Y := y - (H div 2);
end;

procedure TMob.MouseMove(x, y : Integer; m : TMouseButton);
begin
  if Not(isClicked) then exit;
  self.X := x - (W div 2);
  self.Y := y - (H div 2);
end;

procedure TMob.MouseUp(x, y : Integer; m : TMouseButton);
begin
  isClicked := false;
  if Not(isClicked) then exit;
  self.X := x - (W div 2);
  self.Y := y - (H div 2);
end;

end.
